//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
#include <windows.h>
#include <dsound.h>
#include "wavefile.h"

#define MRCHECK(r,str,label)\
    if ((r != MMSYSERR_NOERROR)) { RETAILMSG(1, (TEXT(#str), TEXT(" failed. mr=%08x\r\n"), r)); mr = r; goto label;}


MMRESULT
PlayFile(LPCTSTR pszFilename)
{ MMRESULT mr;
  DWORD dwBufferSize;
  PBYTE pBufferBits = NULL;
  PWAVEFORMATEX pwfx = NULL;
  DWORD dwSlop;
  DWORD dwWait; 
  DWORD dwDuration;
  
    HANDLE hevDone = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (hevDone == NULL) {
        return MMSYSERR_NOMEM;
    }

    mr = ReadWaveFile(pszFilename,&pwfx,&dwBufferSize,&pBufferBits);
    MRCHECK(mr, ReadWaveFile, ERROR_READ);

    dwDuration = dwBufferSize * 1000 / pwfx->nSamplesPerSec;

    RETAILMSG(1, (TEXT("\"%s\" %c%02d %5dHz %6d bytes %5d ms\r\n")
        , pszFilename
        , pwfx->nChannels == 2 ? L'S' : L'M' 
        , pwfx->wBitsPerSample
        , pwfx->nSamplesPerSec
        , dwBufferSize
        , dwDuration
        ));
    
    HWAVEOUT hwo;
    mr = waveOutOpen(&hwo, 0, pwfx, (DWORD) hevDone, NULL, CALLBACK_EVENT);
    MRCHECK(mr, waveOutOpen, ERROR_OPEN);

    WAVEHDR hdr;
    memset(&hdr, 0, sizeof(hdr));
    hdr.dwBufferLength = dwBufferSize;
    hdr.lpData = (char *) pBufferBits;

    mr = waveOutPrepareHeader(hwo, &hdr, sizeof(hdr));
    MRCHECK(mr, waveOutPrepareHeader, ERROR_PLAY);

    mr = waveOutWrite(hwo, &hdr, sizeof(hdr));
    MRCHECK(mr, waveOutWrite, ERROR_PLAY);

    // wait for play + 1 second slop
    dwSlop = 1000;
    dwWait = WaitForSingleObject(hevDone, dwDuration + dwSlop); 
    if (dwWait != WAIT_OBJECT_0) {
        // not much to here, other than issue a warning
        RETAILMSG(1, (TEXT("Timeout waiting for playback to complete\r\n")));
    }

    mr = waveOutUnprepareHeader(hwo, &hdr, sizeof(hdr));
    MRCHECK(mr, waveOutUnprepareHeader, ERROR_PLAY);
    
ERROR_PLAY:
    mr = waveOutClose(hwo);
    MRCHECK(mr, waveOutClose, ERROR_OPEN);

ERROR_OPEN:
    delete [] pBufferBits;
    delete [] pwfx;

ERROR_READ:
    CloseHandle(hevDone);
    return mr;
}




int 
WINAPI WinMain 
    ( HINSTANCE hInstance
    , HINSTANCE hPrevInstance
    , LPTSTR lpCmdLine
    , int nCmdShow
    )
{
    if (lpCmdLine == NULL || *lpCmdLine == 0) {
		RETAILMSG(1, (TEXT("Usage: wavplay filename\r\n")));
		return 1;
    }

    PlayFile(lpCmdLine);
    return 0;
}
